/***************************************************************************
* Copyright (c) 2018 Thomas Hoehn <thomas_hoehn@gmx.net>
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge,
* publish, distribute, sublicense, and/or sell copies of the Software,
* and to permit persons to whom the Software is furnished to do so,
* subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
* OR OTHER DEALINGS IN THE SOFTWARE.
*
* Note: helper for password change dialog, hide signal handling for pam conversation from theme
*
***************************************************************************/

import QtQuick 2.0
import SddmComponents 2.0

Item {

    property string currentPamRequest
    property string currentPamMessage
    property string currentExpiringPamMessage
    property bool requireChangePassword: true

    property bool showExpiringPassword: false

    signal pamFirstRequestHandler
    signal errorHandler
    signal changeCurrentPamRequest(string pamRequest)
    signal changeCurrentPamMessage(string pamMessage)
    signal changeExpiringCurrentPamMessage(string expiringPamMessage)

    Component.onCompleted: {
        if (showExpiringPassword) {
            sddm.enableDelay()
        }

        sddm.enablePwdChange()
    }

    function cancelConnection(){
        sddm.cancelPamConv()
        requireChangePassword = true;
    }

    function responsePamConnection(pwd){
        sddm.pamResponse(pwd)
    }

    Connections {

        target: sddm

        onLoginFailed: {
            errorHandler()
            cancelConnection()
        }

        onPamRequest: {
            if (requireChangePassword) {
                requireChangePassword = false;
                pamFirstRequestHandler()
            }
            currentPamRequest = request.findChangePwdMessage()
            changeCurrentPamRequest(currentPamRequest)
        }


        onPamConvMsg: {
            //Hack for show expiring password message separate from other
            if (/\d/.test(pam_msg) && result == 0) {
                changeExpiringCurrentPamMessage(pam_msg)
            } else {
                currentPamMessage = pam_msg
                changeCurrentPamMessage(currentPamMessage)
            }
        }
    }
}
