## Introduction
SDDM has added processing for changing an outdated password during authorization.
frontend communicates with the backend, which calls pam and processes requests.
From the interface side, this is determined by calling ``PamConversationHelper.qml`` in the SDDM user theme

## Usage
To use the program, you need to add ``PamConversationHelper.qml`` to your project,
for which the server part must take into account that your program supports uploading a request with a password change.

To do this, you need to import SDDM components

``importing SDDM 2.0 components``

Finally, you can add the pam helper itself.
``
PamConversationHelper {
ID: pamconverhelper
}
``
## Functional keys
The theme developer can enable a delay before authorization in case 
it is necessary to display information about an expiring password to the user. 

To do this, when calling ``PamConversationHelper``, 
it is enough to set the ``showExpirePassword = true``

### Server signals

Signals coming from the server side of the daemon

* ``pamConvMsg(pam_msg, result)``
Provides information/errors from the (pam) server dialog for presentation to the user.
The results of the pam function, for example, pam_authentication, are provided for evaluation.

* ``pamRequest()``
A new request from the backend (pam), a user response is required.

### Signals frontend

``PamConversationHelper.qml`` generates signals as data is received from the backend.
Such intermediate processing allows you to flexibly customize custom themes and
get results on the fly, adding the necessary functionality that needs to be performed.

* ``pamFirstRequestHandler()``
Because of this signal leaked once in memory of receiving information about creating a community with pam,
at this point work begins on replacing the software, initializing the application that appears here, thanks to the ``Connections`` in qml.

example
``
Connections {
Target: pamconverhelper

onPamFirstRequestHandler: {
// Initialize the component
}
}
``

* ``Error Handler()``
A signal to stop changing the password due to an error.
After the call, the password change is interrupted, usually used to bind the interface to process the return to authorization.

``
onErrorHandler: {
// Close the password change window, open the authorization window
}
``

* ``changeCurrentPamRequest(pamRequest string)``
The signal appeals to the stock status from PAM,
appeals simultaneously from three requirements:
- current password
- new password
- retype the new password

example
``
Connections {
Target: pamconverhelper

onChangeCurrentPamRequest: function(pamRequest) {
// Show the user what he should enter
}
}
``

* ``changeCurrentPamMessage(pamMessage string)``
The signal appeals to intelligible and formalized messages about pam.
They may contain the reason why the password needs to be changed,
as well as the reasons why the password change was interrupted (for example, if you entered an incorrect password)

example
``
Connections {
Target: pamconverhelper

onChangeCurrentPamMessage: function(pamMessage) {
// Informally, we will remember the data transmitted from the pam
}
}
``

* ``changeExpiringCurrentPamMessage(string expiringPamMessage)``
The alarm is triggered when an expiring password has been detected. 
On the part of the theme developer, you can connect to the signal and 
receive an information message displaying the number of days before the password expires. 

example
``
Connections {
Target: pamconverhelper

onChangeExpiringCurrentPamMessage: function(expiringPamMessage) {
// Processing the output result of an information message
}
}
``

Used in conjunction with the key ``showExpiringPassword = true``

### Ownership

* ``request`` (enter AuthRequest)
Requests from the backend (pam) with messages from pam_conv.

### The answer

Responses from the greeter interface to the backend:

* ``sddm.enablePwdChange()``
The theme tells greeter that it can handle password changes (it has a dialog box and logic as described above).
For those who do not have the option to change the password, this method is not called.
In this case, the pam dialog is simply canceled (otherwise, the pam_conv user session remains there waiting for a password response).
This will keep compatibility with (older) themes that do not yet support expired passwords.

* ``sddm.pamResponse(password)``
Send the password response to the backend (pam), i.e. pam_conv.

* ``sddm.cancelPamConv()``
Cancel the pam dialog using pam_conv.


## New sddm.conf entries

* ``Local file``
The path to the locale.conf file for a specific distribution (for the pam backend) to set the language standard for the PAM dialog
and user session.

* ``RetryLoop``
The PAM retry parameter for changing the password using the pam_pwquality module is usually very low. Set the value
true to ignore the PAM_MAXTRIES error from pam_chauthtok and repeat the password request.
